//
//  FHAgent.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/11/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"

static FHAgent *__loggedInAgent = nil;


@interface FHAgent ()
@property (nonatomic, copy) NSString *password;
@end




@implementation FHAgent

- (id)init
{
    self = [super init];
    if (self) {
        _companies = [NSMutableOrderedSet orderedSet];
    }
    return self;
}


+ (FHAgent *)agentWithIdentifier:(NSNumber *)identifier
{
    return (FHAgent *)[self objectWithIdentifier:identifier];
}

+ (FHAgent *)agentWithEmail:(NSString *)email password:(NSString *)password
{
    FHAgent *agent = [FHAgent agentWithIdentifier:nil];
    agent.email     = email;
    agent.password  = password;
    return agent;
}

+ (FHAgent *)agentWithAccessToken:(NSString *)accessToken
{
    [FHClient sharedClient].token = accessToken;

    FHAgent *agent = [FHAgent agentWithIdentifier:nil];
    agent.accessToken = accessToken;
    return agent;
}




+ (FHAgent *)loggedInAgent
{
    return __loggedInAgent;
}




- (MTPocketRequest *)signupWithFirstName:(NSString *)firstName lastName:(NSString *)lastName
{
    [self logout];
    _firstName  = firstName;
    _lastName   = lastName;
    NSDictionary *body = @{ @"agent" : [self exportDictionary] };
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"agents"
                                                            identifiers:nil
                                                                 method:MTPocketMethodPOST
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:body];
    [request addSuccess:^(MTPocketResponse *response) {
        [self populateWithDictionary:response.body];
        [FHClient sharedClient].token       = _accessToken;
        [FHClient sharedClient].URLToken    = _URLToken;
        __loggedInAgent = self;
    }];

    return request;
}

- (MTPocketRequest *)login
{
    NSDictionary *body = nil;
    if (_email && _password) body = @{ @"email" : _email, @"password" : _password };
    else if (_accessToken) [FHClient sharedClient].token = _accessToken;

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"login"
                                                            identifiers:nil
                                                                 method:MTPocketMethodPOST
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:body];
    [request addSuccess:^(MTPocketResponse *response) {
        [self logout];
        [self populateWithDictionary:response.body];
        [FHClient sharedClient].token       = _accessToken;
        [FHClient sharedClient].URLToken    = _URLToken;
        __loggedInAgent = self;
    }];

    return request;
}

- (void)logout
{
    _accessToken = nil;
    _URLToken = nil;
    [FHClient sharedClient].token           = nil;
    [FHClient sharedClient].URLToken        = nil;
    [FHClient sharedClient].billingToken    = nil;
    __loggedInAgent = nil;
}

- (MTPocketRequest *)fetch
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"agents/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodGET
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [self populateWithDictionary:response.body];
        self.isFetched = YES;
    }];

    return request;
}

- (MTPocketRequest *)save
{
    assert(self.identifier != nil);
    NSDictionary *body = @{ @"agent" : [self exportDictionary] };
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"agents/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodPUT
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:body];
    [request addSuccess:^(MTPocketResponse *response) {
        if (response.body) [self populateWithDictionary:response.body];
        self.isFetched = YES;
    }];

    return request;
}

- (MTPocketRequest *)destroy
{
    assert(self.identifier != nil);
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"agents/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        for (FHCompany *company in _companies) {
            [(NSMutableOrderedSet *)company.agents removeObject:self];
        }
    }];

    return request;
}

- (MTPocketRequest *)dismissNotifications:(NSOrderedSet *)notifications
{
    assert(self.identifier != nil);
    assert(notifications.count > 0);

    NSMutableArray *identifiers = [NSMutableArray array];
    [identifiers addObject:self.identifier];
    for (FHNotification *notification in notifications) {
        [identifiers addObject:notification.identifier];
    }

    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"agents/:id/notifications/:id"
                                                            identifiers:@[self.identifier, identifiers]
                                                                 method:MTPocketMethodPUT
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        for (FHNotification *notification in notifications) {
            [notification.company.notifications removeObject:notification];
        }
    }];

    return request;
}





- (void)setPassword:(NSString *)password
{
    _password = password;
}

- (NSString *)fullName
{
    return [NSString stringWithFormat:@"%@ %@", _firstName, _lastName];
}







#pragma mark - Protected

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    FAVOR_LEFT(_accessToken,    [FHObject importString:dictionary[@"access_token"]]);
    FAVOR_LEFT(_URLToken,       [FHObject importString:dictionary[@"url_token"]]);

    FAVOR_LEFT(_firstName,      [FHObject importString:dictionary[@"first_name"]]);
    FAVOR_LEFT(_lastName,       [FHObject importString:dictionary[@"last_name"]]);
    FAVOR_LEFT(_email,          [FHObject importString:dictionary[@"email"]]);

    [self populateAssociatedObjects:&_companies withDictionaries:dictionary[@"companies"] createObjectBlock:^id(NSDictionary *dict) {
        FHCompany *company = [FHCompany companyWithIdentifier:dict[@"id"] agent:self];
        [(NSMutableOrderedSet *)company.agents addObject:self];
        return company;
    }];

    if ([_companies count] > 0 && !_currentCompany) {
        _currentCompany = _companies[0];
    }
    
    [super populateWithDictionary:dictionary];
}




#pragma mark - Private

- (NSDictionary *)exportDictionary
{
    [self exportString:_firstName   as:@"first_name"];
    [self exportString:_lastName    as:@"last_name"];
    [self exportString:_email       as:@"email"];
    [self exportString:_password    as:@"password"];
    return [super exportDictionary];
}




@end

