//
//  FHCompany.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/14/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"

// is b in the bitmask a
#define IN_MASK(a, b) ((a & b) == b)


@interface FHCompany ()
@property (nonatomic, copy             ) NSString      *identifier;
@property (nonatomic, strong, readwrite) FHAgent       *creator;
@property (nonatomic, strong, readwrite) FHRemoteArray *notifications;
@property (nonatomic, strong, readwrite) FHRemoteArray *twitterAccounts;
@property (nonatomic, strong, readwrite) FHRemoteArray *facebookAccounts;
@property (nonatomic, strong, readwrite) FHRemoteArray *emailAccounts;
@end


@implementation FHCompany

- (id)init
{
    self = [super init];
    if (self) {
        _fetchAutomatically = YES;
        _agents             = [NSMutableOrderedSet orderedSet];
        _agentInvites       = [NSMutableOrderedSet orderedSet];
        _tags               = [NSMutableOrderedSet orderedSet];
        _cannedResponses    = [NSMutableOrderedSet orderedSet];

        // set billing defaults
        _billingRate            = 0;
        _isGracePeriodOver      = NO;
        _daysLeftInGracePeriod  = 0;
        _isCurrent              = NO;
        _hasSuccessfulBilling   = NO;
    }
    return self;
}

+ (FHCompany *)companyWithIdentifier:(NSNumber *)identifier agent:(FHAgent *)agent
{
    FHCompany *company = (FHCompany *)[self objectWithIdentifier:identifier];
    [(NSMutableOrderedSet *)company.agents addObject:agent];
    return company;
}

+ (FHCompany *)companyWithAgent:(FHAgent *)agent title:(NSString *)title
{
    assert(agent != nil);
    FHCompany *company = [FHCompany companyWithIdentifier:nil agent:agent];
    company.creator = agent;
    company.title   = title;
    return company;
}



- (MTPocketRequest *)fetch
{
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"companies/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodGET
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        [self populateWithDictionary:response.body];
        self.isFetched = YES;
    }];

    return request;
}

- (MTPocketRequest *)save
{
    MTPocketRequest *request = nil;
    if (!self.identifier) {
        NSDictionary *body = @{ @"company" : [self exportDictionary] };
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"agents/:agent_id/companies"
                                               identifiers:@[_creator.identifier]
                                                    method:MTPocketMethodPOST
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
        [request addSuccess:^(MTPocketResponse *response) {
            [self populateWithDictionary:response.body];
            _creator.currentCompany = self;
            self.isFetched = YES;
        }];
    }
    else {
        NSDictionary *body = @{ @"company" : [self exportDictionary] };
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"companies/:id"
                                               identifiers:@[self.identifier]
                                                    method:MTPocketMethodPUT
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:body];
    }
    return request;
}

- (MTPocketRequest *)forceChannelsFetch
{
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"companies/:id/force_channels_fetch"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodPUT
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    return request;
}

- (MTPocketRequest *)destroy
{
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                                  route:@"companies/:id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodDELETE
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addSuccess:^(MTPocketResponse *response) {
        FHAgent *agent = [FHAgent loggedInAgent];
        if ([agent.currentCompany isEqual:self]) {
            agent.currentCompany = [agent.companies lastObject];
        }
        for (FHAgent *agent in _agents) {
            [(NSMutableOrderedSet *)agent.companies removeObject:self];
        }
    }];
    return request;
}




#pragma mark - Remote Arrays

- (FHRemoteArray *)customersMatchingResolvedFilter:(FHCustomerResolvedFilter)resolvedFilter
                                     channelFilter:(FHCustomerChannelFilter)channelFilter
                                      searchString:(NSString *)searchString
                                              sort:(FHCustomerSort)sort
{
    if (!self.identifier) return nil;

    NSMutableDictionary *params = [NSMutableDictionary dictionary];
    params[@"filter"]           = [self stringFromResolvedFilterEnum:resolvedFilter];
    params[@"channel"]          = [self stringFromChannelFilterEnum:channelFilter];
    params[@"sort"]             = [self stringFromSortEnum:sort];
    if (searchString)
        params[@"search_text"]  = searchString;

    return [FHRemoteArray remoteArrayWithPath:@"companies/:id/customers" identifiers:@[self.identifier] params:params createObjectBlock:^id(NSDictionary *dict) {
        return [FHCustomer customerWithIdentifier:dict[@"id"] company:self];
    }];
}

- (FHRemoteArray *)notifications
{
    if (!self.identifier) return nil;
    if (!_notifications) {
        _notifications = [FHRemoteArray remoteArrayWithPath:@"companies/:id/notifications" identifiers:@[self.identifier] params:nil createObjectBlock:^id(NSDictionary *dict) {
            return [FHNotification notificationWithIdentifier:dict[@"id"] company:self];
        }];
    }
    return _notifications;
}

- (FHRemoteArray *)twitterAccounts
{
    if (!self.identifier) return nil;
    if (!_twitterAccounts) {
        _twitterAccounts = [FHRemoteArray remoteArrayWithPath:@"companies/:id/twitter_accounts" identifiers:@[self.identifier] params:nil createObjectBlock:^id(NSDictionary *dict) {
            return [FHTwitterAccount twitterAccountWithIdentifier:dict[@"id"] company:self];
        }];
    }
    return _twitterAccounts;
}

- (FHRemoteArray *)facebookAccounts
{
    if (!self.identifier) return nil;
    if (!_facebookAccounts) {
        _facebookAccounts = [FHRemoteArray remoteArrayWithPath:@"companies/:id/facebook_accounts" identifiers:@[self.identifier] params:nil createObjectBlock:^id(NSDictionary *dict) {
            return [FHFacebookAccount facebookAccountWithIdentifier:dict[@"id"] company:self];
        }];
    }
    return _facebookAccounts;
}

- (FHRemoteArray *)emailAccounts
{
    if (!self.identifier) return nil;
    if (!_emailAccounts) {
        _emailAccounts = [FHRemoteArray remoteArrayWithPath:@"companies/:id/email_accounts" identifiers:@[self.identifier] params:nil createObjectBlock:^id(NSDictionary *dict) {
            return [FHEmailAccount emailAccountWithIdentifier:dict[@"id"] company:self];
        }];
    }
    return _emailAccounts;
}




#pragma mark - Add/Remove Agent

- (MTPocketRequest *)addAgent:(FHAgent *)agent
{
    assert(agent.identifier != nil);
    assert(self.identifier != nil);

    MTPocketRequest *request = nil;

    if (![agent isAssociated:_agents]) {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"companies/:company_id/agents/:id"
                                               identifiers:@[self.identifier, agent.identifier]
                                                    method:MTPocketMethodPUT
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:nil];
        [request addSuccess:^(MTPocketResponse *response) {
            [(NSMutableOrderedSet *)_agents addObject:agent];
        }];
    }
    return request;
}

- (MTPocketRequest *)removeAgent:(FHAgent *)agent
{

    assert(agent.identifier != nil);
    assert(self.identifier != nil);

    MTPocketRequest *request = nil;

    if ([agent isAssociated:_agents]) {
        request = [[FHClient sharedClient] requestToServer:FHServerAPI
                                                     route:@"companies/:company_id/agents/:id"
                                               identifiers:@[self.identifier, agent.identifier]
                                                    method:MTPocketMethodDELETE
                                                      page:-1
                                                   perPage:-1
                                                    params:nil
                                                      body:nil];
        [request addSuccess:^(MTPocketResponse *response) {
            [(NSMutableOrderedSet *)_agents removeObject:agent];
        }];
    }
    return request;
}






#pragma mark - Billings

- (MTPocketRequest *)fetchBillingInfo
{
    assert(self.identifier != nil);
    [FHClient sharedClient].billingToken = _token;
    MTPocketRequest *request = [[FHClient sharedClient] requestToServer:FHServerBilling
                                                                  route:@"entities/:entity_id"
                                                            identifiers:@[self.identifier]
                                                                 method:MTPocketMethodGET
                                                                   page:-1
                                                                perPage:-1
                                                                 params:nil
                                                                   body:nil];
    [request addComplete:^(MTPocketResponse *response) {
        NSDictionary *body = response.body;
        if (body) {
            if (body[@"credit_card"]) {
                _creditCard = [FHCreditCard creditCardWithCompany:self];
                [_creditCard populateWithDictionary:body[@"credit_card"]];
            }

            _billingEmail           = [FHObject importString:body[@"email"]];
            _billingRate            = [FHObject importFloat:body[@"rate"]] / 100.0;
            _trialExpirationDate    = [FHObject importDate:body[@"free_trial_expiration_date"]];
            _nextBillingDate        = [FHObject importDate:body[@"next_bill_date"]];
            _isGracePeriodOver      = [FHObject importBool:body[@"grace_period_over"]];
            _daysLeftInGracePeriod  = [FHObject importInteger:body[@"days_left_in_grace_period"]];
            _isCurrent              = [FHObject importBool:body[@"current"]];
            _hasSuccessfulBilling   = [FHObject importBool:body[@"has_successful_billing"]];
        }
        else {
            _billingEmail           = [FHAgent loggedInAgent].email;
            _billingRate            = 8.00;
            _trialExpirationDate    = [NSDate dateWithTimeIntervalSinceNow:60*60*24*14];
            _nextBillingDate        = nil;
            _isGracePeriodOver      = NO;
            _daysLeftInGracePeriod  = -1;
            _isCurrent              = NO;
            _hasSuccessfulBilling   = NO;
        }
    }];
    return request;
}






#pragma mark - Protected

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _title              = [FHObject importString:dictionary[@"title"]];
    _token              = [FHObject importString:dictionary[@"token"]];
    _fetchAutomatically = [FHObject importBool:dictionary[@"fetch_automatically"]];
    _lastFetchAt        = [FHObject importDate:dictionary[@"last_fetch_at"]];
    _unresolvedCount    = [FHObject importInteger:dictionary[@"unresolved_count"]];
    _numberOfAccounts   = [FHObject importInteger:dictionary[@"number_of_accounts"]];

    [self populateAssociatedObjects:&_agents withDictionaries:dictionary[@"agents"] createObjectBlock:^id(NSDictionary *dict) {
        FHAgent *agent = [FHAgent agentWithIdentifier:dict[@"id"]];
        [(NSMutableOrderedSet *)agent.companies addObject:self];
        return agent;
    }];

    [self populateAssociatedObjects:&_agentInvites withDictionaries:dictionary[@"agent_invites"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHAgentInvite agentInviteWithIdentifier:dict[@"id"] company:self];
    }];

    [self populateAssociatedObjects:&_tags withDictionaries:dictionary[@"tags"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHTag tagWithIdentifier:dict[@"id"] company:self];
    }];

    [self populateAssociatedObjects:&_cannedResponses withDictionaries:dictionary[@"canned_responses"] createObjectBlock:^id(NSDictionary *dict) {
        return [FHCannedResponse cannedResponseWithIdentifier:dict[@"id"] company:self];
    }];

    [super populateWithDictionary:dictionary];

    [FHClient sharedClient].billingToken = _token;
}



#pragma mark - Private

- (NSDictionary *)exportDictionary
{
    [self exportString:_title            as:@"title"];
    [self exportBool:_fetchAutomatically as:@"fetch_automatically"];
    return [super exportDictionary];
}

- (NSString *)stringFromResolvedFilterEnum:(FHCustomerResolvedFilter)resolvedFilter
{
    if (resolvedFilter == FHCustomerResolvedFilterEverything) return @"everything";
    if (resolvedFilter == FHCustomerResolvedFilterUnresolved) return @"unresolved";
    return @"everything";
}

- (NSString *)stringFromChannelFilterEnum:(FHCustomerChannelFilter)channelFilter
{
    NSMutableArray *channels = [NSMutableArray array];
    if (IN_MASK(channelFilter, FHCustomerChannelFilterTwitter))     [channels addObject:@"twitter"];
    if (IN_MASK(channelFilter, FHCustomerChannelFilterFacebook))    [channels addObject:@"facebook"];
    if (IN_MASK(channelFilter, FHCustomerChannelFilterEmail))       [channels addObject:@"email"];
    return [channels componentsJoinedByString:@","];
}

- (NSString *)stringFromSortEnum:(FHCustomerSort)sort
{
    if (sort == FHCustomerSortNewestFirst)  return @"newest_first";
    if (sort == FHCustomerSortOldestFirst)  return @"oldest_first";
    return @"newest_first";
}

- (NSString *)channelNameWithPrefix:(NSString *)prefix
{
    return [NSString stringWithFormat:@"%@_%@", prefix, _token];
}



@end










