//
//  FHObject.m
//  FirehoseCocoa
//
//  Created by Adam Kirk on 1/21/13.
//  Copyright (c) 2013 Mysterious Trousers. All rights reserved.
//

#import "private.h"
#import "NSString+HTML.h"


static NSMutableArray *__cachedObjects = nil;


@interface FHObject ()
@property (nonatomic, copy) NSMutableDictionary *exportDictionary;
@end


@implementation FHObject

- (id)init
{
    self = [super init];
    if (self) {
        _exportDictionary   = [NSMutableDictionary dictionary];
        _isFetched          = NO;
        [__cachedObjects addObject:self];
    }
    return self;
}

- (id)initWithIdentifier:(NSNumber *)identifier
{
    self = [self init];
    if (self) {
        _identifier = identifier;
    }
    return self;
}

+ (FHObject *)objectWithIdentifier:(NSNumber *)identifier
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        __cachedObjects = [NSMutableArray array];
    });

    // if it has an identifier, look if it's already in the cache
    if (identifier) {
        for (FHObject *object in __cachedObjects) {
            if (object.identifier && [object isKindOfClass:self] && [identifier isEqualToNumber:object.identifier]) {
                return object;
            }
        }
    }

    FHObject *object = [[self alloc] initWithIdentifier:identifier];

    return object;
}

- (BOOL)isSaved
{
    return _identifier != nil;
}

- (void)purge
{
    [__cachedObjects removeObject:self];
}


#pragma mark - Overrides

- (BOOL)isEqual:(FHObject *)object
{
    // In this library, there is no reason why two unidentified objects should exits in a set together
    // One might if it's waiting to save and it's identifier will be returned by the server.
    BOOL sameClass      = [object isKindOfClass:[self class]];
    BOOL bothAnonymous  = object.identifier == nil && self.identifier == nil;
    BOOL sameIds        = self.identifier && object.identifier && [self.identifier isEqualToNumber:object.identifier];
    return sameClass && !bothAnonymous && sameIds;
}

- (NSUInteger)hash
{
    return [_identifier unsignedLongValue];
}




#pragma mark - Associations

- (BOOL)isAssociated:(NSOrderedSet *)associated
{
    return [associated containsObject:self];
}

- (void)populateWithDictionary:(NSDictionary *)dictionary
{
    _identifier = [FHObject importNumber:dictionary[@"id"]];
    _createdAt  = [FHObject importDate:dictionary[@"created_at"]];
}

- (void)populateAssociatedObjects:(NSOrderedSet *__strong *)associatedObjects withDictionaries:(NSArray *)dictionaries createObjectBlock:(id (^)(NSDictionary *dict))createObjectBlock
{
    if ([dictionaries isKindOfClass:[NSArray class]]) {
        *associatedObjects = [NSMutableOrderedSet orderedSet];
        for (NSDictionary *dict in dictionaries) {
            FHObject *obj = createObjectBlock(dict);
            [obj populateWithDictionary:dict];
            [(NSMutableOrderedSet *)*associatedObjects addObject:obj];
        }
    }
}

- (void)addToAssociatedObjects:(NSOrderedSet *)associatedObjects withDictionary:(NSDictionary *)dictionary createObjectBlock:(id (^)(NSDictionary *dict))createObjectBlock
{
    if ([dictionary isKindOfClass:[NSDictionary class]] || dictionary == nil) {
        FHObject *obj = createObjectBlock(dictionary);
        if (dictionary) [obj populateWithDictionary:dictionary];
        [(NSMutableOrderedSet *)associatedObjects addObject:obj];
    }
}

- (void)populateAssociatedObject:(FHObject *__strong *)associatedObject withDictionary:(NSDictionary *)dictionary createObjectBlock:(id (^)(NSDictionary *))createObjectBlock
{
    *associatedObject = nil;
    if ([dictionary isKindOfClass:[NSDictionary class]] && [dictionary count] > 0) {
        FHObject *object = createObjectBlock(dictionary);
        [object populateWithDictionary:dictionary];
        *associatedObject = object;
    }
}

- (void)populateAssociatedObject:(FHObject *__strong *)associatedObject withIdentifier:(NSNumber *)identifier createObjectBlock:(id (^)(NSNumber *))createObjectBlock
{
    *associatedObject = nil;
    identifier = NILL(identifier);
    if (identifier) {
        *associatedObject = createObjectBlock(identifier);
        (*associatedObject)->_identifier = identifier;
    }
}




#pragma mark - Value Importers

+ (int)importEnum:(NSString *)string usingMap:(NSDictionary *)map
{
    id key = NILL(string);
    if (key) {
        id val = map[string];
        if (val)
            return [val intValue];
    }
    return 0;
}

+ (NSString *)importString:(NSString *)string
{
    return [NILL(string) stringByDecodingHTMLEntities];
}

+ (NSNumber *)importNumber:(NSNumber *)number
{
    return NILL(number);
}

+ (BOOL)importBool:(NSNumber *)value
{
    return NILL(value) ? [value boolValue] : NO;
}

+ (NSInteger)importInteger:(NSNumber *)number
{
    return NILL(number) ? [number integerValue] : 0;
}

+ (CGFloat)importFloat:(NSNumber *)number
{
    return NILL(number) ? [number floatValue] : 0.0;
}

+ (NSDate *)importDate:(NSString *)string
{
    return NILL(string) ? [NSDate mt_dateFromISOString:string] : nil;
}

+ (NSURL *)importURL:(NSString *)string
{
    return NILL(string) ? [NSURL URLWithString:string] : nil;
}






#pragma mark - Value Exporters

- (void)exportEnum:(int)enumValue               as:(NSString *)key      usingMap:(NSDictionary *)map
{
    id object = [map allKeysForObject:@(enumValue)];
    if ([object count] > 0)
        _exportDictionary[key] = [object lastObject];
}

- (void)exportString:(NSString *)stringValue    as:(NSString *)key
{
    if (stringValue)
        _exportDictionary[key] = stringValue;
}

- (void)exportNumber:(NSNumber *)numberValue    as:(NSString *)key
{
    if (numberValue)
        _exportDictionary[key] = numberValue;
}

- (void)exportBool:(BOOL)boolValue              as:(NSString *)key
{
    _exportDictionary[key] = @(boolValue);
}

- (void)exportInteger:(NSInteger)integerValue   as:(NSString *)key
{
    _exportDictionary[key] = @(integerValue);
}

- (void)exportDate:(NSDate *)dateValue          as:(NSString *)key
{
    if (dateValue)
        _exportDictionary[key] = [dateValue mt_stringFromDateWithISODateTime];
}

- (void)exportURL:(NSURL *)URLValue             as:(NSString *)key
{
    if (URLValue)
        _exportDictionary[key] = [URLValue absoluteString];
}



#pragma mark - Export

- (NSDictionary *)exportDictionary
{
    NSDictionary *dict = [_exportDictionary copy];
    [_exportDictionary removeAllObjects];
    return dict;
}






@end




